import gnu.io.SerialPortEventListener;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.EventListener;

/**
 * 
 */

/**
 * @author Christophe HAVARD
 *
 */
public class Glove implements IGlove {

	
	/** Buffered input stream from the port */
	private InputStream input = null;
	/** The output stream to the port */
	private OutputStream output;
	private SerialPortEventListener serialPortEventListener;
	private boolean connected = false;
	private final ICommands commands = new SimpleCommand();
	
	public Glove(final InputStream inputStream, final OutputStream outputStream, final SerialPortEventListener serialPortEventListener) {
		this.input = inputStream;
		this.output = outputStream;
		this.serialPortEventListener = serialPortEventListener;
	}
	/**
	 * Return the input stream from the COM port
	 */
	public InputStream getInputStream() {
		return this.input;
	}
	
	/**
	 * Return the output stream from the COM port
	 */
	public OutputStream getOutputStream() {
		return this.output;
	}
	
	/**
	 * Return the serial port event listener of the current glove. 
	 * This listener allows the glove to receive input data from the hardware.
	 * @return The serial port listener of the current glove.
	 */
	@Override
	public EventListener getInputEventListener() {
		return this.serialPortEventListener;
	}

	@Override
	public void read() throws IOException {
		this.input.read();
		
	}
	@Override
	public void write(byte[] b) throws IOException {
		this.output.write(b);
		this.output.flush();//ensure that the buffer content is empty.
		System.out.println("Trame envoyée : " + Arrays.toString(b));
	}
	@Override
	public boolean isConnected() {
		return this.connected ;
	}
	
	public void vibrate(FINGERS finger, int duration){
		byte[] trame = null;
		if(isConnected()){
			switch(finger){
			case THUMB:
				trame = commands.getThumb(duration);
				break;
			case FOREFINGER:
				trame = commands.getForeFinger(duration);
				break;
			case MIDDLEFINGER:
				trame = commands.getMiddleFinger(duration);
				break;
			case RINGFINGER:
				trame = commands.getRingFinger(duration);
				break;
			case LITTLEFINGER:
				trame = commands.getLittleFinger(duration);
				break;
			case ALL:
				trame = commands.getAllFingers(duration);
				break;
			}
			
			try {
				this.write(trame);
			} catch (IOException e) {
				System.err.println("Probleme avec l'envoi de la trame sur le serial port : " + e);
			}
		} else {
			System.out.println("WARNING : THE GLOVE IS NOT CONNECTED");
		}
	}
	@Override
	public void setConnected(boolean b) {
		this.connected = b;
	}
	@Override
	public void vibrate(FINGERS finger, int duration, int level) {
		byte[] trame = null;
		if(isConnected()){
			switch(finger){
			case THUMB:
				trame = commands.getThumb(duration, level);
				break;
			case FOREFINGER:
				trame = commands.getForeFinger(duration, level);
				break;
			case MIDDLEFINGER:
				trame = commands.getMiddleFinger(duration, level);
				break;
			case RINGFINGER:
				trame = commands.getRingFinger(duration, level);
				break;
			case LITTLEFINGER:
				trame = commands.getLittleFinger(duration, level);
				break;
			case ALL:
				trame = commands.getAllFingers(duration, level);
				break;
			}
			
			try {
				this.write(trame);
			} catch (IOException e) {
				System.err.println("Probleme avec l'envoi de la trame sur le serial port : " + e);
			}
		} else {
			System.out.println("WARNING : THE GLOVE IS NOT CONNECTED");
		}
		
	}
	



}
