import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Enumeration;

/**
 * 
 */

/**
 * @author Christophe HAVARD
 *
 */
public class GloveFactory {
	
	/** Milliseconds to block while waiting for port open */
	final static int TIME_OUT = 2000;
	/** Default bits per second for COM port. */
	final static int DATA_RATE = 9600;
	/**
	 * Represent the unique serial port connected to the real glove
	 */
	private static SerialPort serialPort;

	public static IGlove getNewGlove(final String port) throws InterruptedException{
		/** The port we're normally going to use. */
		final String PORT_NAMES[] = { 
			//				"/dev/tty.usbserial-A9007UX1", // Mac OS X
			//				"/dev/ttyUSB0", // Linux
			port, // Window
		};


			CommPortIdentifier portId = null;
			@SuppressWarnings("unchecked")
			Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();

			// iterate through, looking for the port
			while (portEnum.hasMoreElements()) {
				CommPortIdentifier currPortId = portEnum.nextElement();
				for (String portName : PORT_NAMES) {
					// the second part of the test connect the specified COM port to this new instance of a glove
					if (currPortId.getName().equals(portName) && portName.equalsIgnoreCase(port)) {
						portId = currPortId;
						break;
					}
				}
			}

			if (portId == null) {
				System.out.println("Could not find COM port.");
				return null;
			}

			try {
				// open serial port, and use GLOVE_COM_PORT for the app name.
				serialPort = (SerialPort) portId.open("GLOVE_" + port,TIME_OUT);

				// set port parameters
				serialPort.setSerialPortParams(
						DATA_RATE,
						SerialPort.DATABITS_8,
						SerialPort.STOPBITS_1,
						SerialPort.PARITY_NONE);
			} catch (Exception e) {
				System.err.println(e.toString());
			}
			
			// instanciate a new glove through its implementation
			IGlove glove = null;
			try {
				glove = new Glove(
					// the input stream
					serialPort.getInputStream(),
					// the output stream
					serialPort.getOutputStream(),
					new SerialPortEventListener() {
						
						@Override
						public void serialEvent(SerialPortEvent oEvent) {
//							if (oEvent.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
//								try {
//									int available = input.available();
//									byte chunk[] = new byte[available];
//									input.read(chunk, 0, available);
//									
//									// Displayed results are codepage dependent
//									//System.out.print(new String(chunk) + "\n");
//								} catch (Exception e) {
//									System.err.println(e.toString());
//								}
//							}
//							// Ignore all the other eventTypes, but you should consider the other ones.
//							
						}
					}
				);
				// add event listeners
				serialPort.addEventListener((SerialPortEventListener) glove.getInputEventListener());
				serialPort.notifyOnDataAvailable(true);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			System.out.println("Waiting for Arduino initialization...");
			Thread.sleep(1500);
			System.out.println("Arduino ready!");
			glove.setConnected(true);
			return glove;
			
		}
	/**
	 * Return a FAKE glove where the I/O comes from the console.
	 * @return a new instance of IGlove
	 */
	public static IGlove getNewFakeGlove(){
		// instanciate a new glove through its implementation
			IGlove glove = null;
			byte[] inputBuffer = new byte[0];// a default fake value
			try {
				
				glove = new FakeGlove(
					// the input stream
					new ByteArrayInputStream(inputBuffer),
					// the output stream
					new ByteArrayOutputStream()
				);

			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			return glove;
	}
	
	/**
	 * This should be called when you stop using the port.
	 * This will prevent port locking on platforms like Linux.
	 */
	public static synchronized void destroyGlove() {

		if (serialPort != null) {
			try {
				serialPort.getOutputStream().flush();
				Thread.sleep(1500);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			serialPort.removeEventListener();
			serialPort.close();
			System.out.println("serial port " + serialPort.getName() + " is now closed");
		}
	}
}
